// Gary Yngve 10/30/2005
import java.io.*;
import java.util.*;

public class MineWindow {
    private static final int MINE = 1;
    private static final int UNKNOWN = 0;
    private static final int NOTMINE = -1;
    private static final int NOTASK = -42;
    private int[][] asks, mines;

// not the best style making these public, but i'm running low on time
    public static Set markSet = new HashSet(), exploreSet = new HashSet();

// asks are the 3x3 window centered at (0,0) holding a number representing the
// number of neighboring mines, or NOTASK if not askable
// mines are the 5x5 window centered at (0,0) holding if there is a MINE there,
// if there is NOT a MINE there, or UNKNOWN
    public MineWindow() {
        asks = new int[3][3];
        mines = new int[5][5];
    }

    public MineWindow(MineBoard b, int i, int j) {
        this(b,i,j,1,1);
    }

// makes a MineWindow of the board centered at (i,j) such that
// the square to be decided is at (-1,-1) but originally at (-dx, -dy)
    public MineWindow(MineBoard b, int i, int j, int dx, int dy) {
        this();
        int mm,nn;
        mm=-2;
        for(int m = -2*dx; m != 3*dx; m+=dx) { // ugly code for the rotations
            nn=-2;
            for(int n = -2*dy; n != 3*dy; n+=dy) {
                int x = i+m, y = j+n;
                if (b.isMarked(x,y))
                    setMines(mm,nn,MINE);
                else if (b.askable(x,y) || b.isOOB(x,y))
                    setMines(mm,nn,NOTMINE);
                else setMines(mm,nn,UNKNOWN);
            nn++;}mm++;} // gross!
        mm=-1;
        for(int m = -1*dx; m != 2*dx; m+=dx) {
            nn=-1;
            for(int n = -1*dy; n != 2*dy; n+=dy) {
                int x = i+m, y = j+n;
                if (b.askable(x,y))
                  setAsks(mm,nn,b.ask(x,y));
                else
                  setAsks(mm,nn,NOTASK);
            nn++;}mm++;}
    }

// zero-center the coords
    public int getAsks(int i, int j) {
        return asks[i+1][j+1];
    }

    public int getMines(int i, int j) {
        return mines[i+2][j+2];
    }

    public void setAsks(int i, int j, int val) {
        asks[i+1][j+1] = val;
    }

    public void setMines(int i, int j, int val) {
        mines[i+2][j+2] = val;
    }

// read a Set of MineWindows, returning the Set
    public static Set read(InputStream in) {
        Set s = new HashSet();
        Scanner sc = new Scanner(in);
        if (!sc.hasNextInt()) throw new RuntimeException("bad format");
        int n = sc.nextInt();
        for(int i = 0; i < n; i++) {
            MineWindow mw = new MineWindow();
            mw.readOne(sc);
            s.add(mw);
        }
        return s;
    }

// write a Set of MineWindows to file
    public static void write(Set s, PrintStream out) {
        out.println(s.size());
        for(Iterator i = s.iterator(); i.hasNext(); ) {
            ((MineWindow)i.next()).writeOne(out);
        }
    }

// read one MineWindow, filling in this
    protected void readOne(Scanner sc) {
        for(int m = -2; m <= 2; m++)
            for(int n = -2; n <= 2; n++) {
                if (!sc.hasNextInt())
                    throw new RuntimeException("bad format1 "+m+","+n);
                mines[m+2][n+2] = sc.nextInt();
            }
        for(int m = -1; m <= 1; m++)
            for(int n = -1; n <= 1; n++) {
                if (!sc.hasNextInt())
                    throw new RuntimeException("bad format2 "+m+","+n);
                asks[m+1][n+1] = sc.nextInt();
            }
    }

// write one MineWindow to file
    protected void writeOne(PrintStream out) {
        for(int m = -2; m <= 2; m++)
            for(int n = -2; n <= 2; n++)
                out.print(mines[m+2][n+2]+" ");
        out.println();
        for(int m = -1; m <= 1; m++)
            for(int n = -1; n <= 1; n++)
                out.print(asks[m+1][n+1]+" ");
        out.println();
    }

    public String toString() {
        String str="";
        for(int j = -2; j <=2; j++) {
            for(int i = -2; i <=2; i++) {
                if (i>1 || i<-1 || j>1 || j<-1 || getAsks(i,j)==NOTASK) {
                    str += getMines(i,j)==MINE?'*':getMines(i,j)==NOTMINE?'-':'?';
                } else {
                    str += getAsks(i,j);
                }
            } str += '\n';
        }
        return str;
    }

    public boolean equals(Object o) {
        if (!(o instanceof MineWindow)) return false;
        MineWindow mw = (MineWindow)o;
        for(int m = -2; m <= 2; m++)
            for(int n = -2; n <= 2; n++)
                if (mw.getMines(m,n) != this.getMines(m,n))
                    return false;
        for(int m = -1; m <= 1; m++)
            for(int n = -1; n <= 1; n++)
                if (mw.getAsks(m,n) != this.getAsks(m,n))
                    return false;
        return true;
    }

    public int hashCode() {
        throw new RuntimeException("not implemented");
    }

}
